/** Represents a scope within a TableOfVars instance */
function TableOfVarsScope(
    parentTable,    // TableOfVarsAbstract instance containing this instance
    pointer         // Memory location of instance beeing this
) {
    TableOfVars.parent.call(this, parentTable);
    this.pointer = pointer;
    this.nodeData = parentTable.addTbody();
    this._buildFirstRow();
}
TableOfVarsScope.parent = TableOfVarsAbstract;
TableOfVarsScope.prototype = Object.create(TableOfVarsScope.parent.prototype);
TableOfVarsScope.prototype.constructor = TableOfVarsScope;

/** Memory location of instance beeing this */
TableOfVarsInstance.prototype.pointer = undefined;

/**
* Add title row of scope ("this = …")
* @return undefined
*/
TableOfVarsScope.prototype._buildFirstRow = function() {
    const doc = this.getDocument();
    const tr = doc.createElement('tr');
    const code = ScoTools.createHtml('code', ScoTools.createHtml('th', tr, { colspan: 3 }));
    ScoTools.createHtml('span', code, { 'class': 'hljs-keyword' }, 'this');
    ScoTools.createHtml(undefined, code, null, ' ');
    ScoTools.createHtml('span', code, { 'class': 'hljs-operator' }, '=');
    ScoTools.createHtml(undefined, code, null, ' ');
    if (Number.isFinite(this.pointer))
        ScoTools.createHtml('span', code, { 'class': 'address' }, this.pointerToAddress(this.pointer));
    else
        ScoTools.createHtml('span', code, { 'class': 'hljs-number' }, 'null');
    ScoTools.createHtml('td', tr).appendChild(this.createCellContentEdit(doc, () => this.tryRemove(), 'Kompletten Sichtbarkeitsbereich'));
    this.nodeData.appendChild(tr);
}

/**
* Try to remove this scope
* @return undefined
*/
TableOfVarsScope.prototype.tryRemove = function() {
    if (this != this.scopes[this.scopes.length - 1]) {
        this._addfailed('Nur der aktuellste Sichtbarkeitsbereich kann entfernt werden');
        return;
    }
    this.remove();
}

/**
* Force remove this scope
* @return undefined
*/
TableOfVarsScope.prototype.remove = function() {
    this.removeScope(this, this.nodeData);
}

/**
* Convert contained data to JSON
* @return Object    JSON representation of contained data
*/
TableOfVarsScope.prototype.toJSON = function() {
    return { ref: Number.isFinite(this.pointer) ? this.pointer : null, vars: TableOfVarsScope.parent.prototype.toJSON.call(this) };
}
